@@ -5,10 +5,12 @@ sieve <- function(x, ...)
   UseMethod("sieve")
 
 sieve.formula <-
-function(formula, data = NULL, ..., main = NULL, subset = NULL)
+function(formula, data = NULL, ..., main = NULL, sub = NULL, subset = NULL)
 {
   if (is.logical(main) && main)
     main <- deparse(substitute(data))
+  else if (is.logical(sub) && sub)
+    sub <- deparse(substitute(data))
   
   m <- match.call(expand.dots = FALSE)
   edata <- eval(m$data, parent.frame())
@@ -34,7 +36,7 @@ function(formula, data = NULL, ..., main = NULL, subset = NULL)
       }
       dat <- margin.table(dat, ind)
     }
-    sieve.default(dat, main = main,
+    sieve.default(dat, main = main, sub = sub,
                    condvars = if (is.null(condind)) NULL else match(condnames, names(dimnames(dat))), ...)
   } else {
     tab <- if ("Freq" %in% colnames(data))
@@ -44,17 +46,20 @@ function(formula, data = NULL, ..., main = NULL, subset = NULL)
       xtabs(formula(paste("~", paste(c(condnames, varnames), collapse = "+"))),
             data = data, subset = subset)
     
-    sieve.default(tab, main = main, ...)
+    sieve.default(tab, main = main, sub = sub, ...)
   }
 }
 
 sieve.default <- function(x, condvars = NULL, gp = NULL,
                           shade = NULL, legend = FALSE,
-                          split_vertical = FALSE, direction = NULL,
+                          split_vertical = NULL, direction = NULL,
                           spacing = NULL, spacing_args = list(),
-                          sievetype = c("observed","expected"), main = NULL, ...) {
+                          sievetype = c("observed","expected"),
+                          main = NULL, sub = NULL, ...) {
   if (is.logical(main) && main)
     main <- deparse(substitute(x))
+  else if (is.logical(sub) && sub)
+    sub <- deparse(substitute(x))
   sievetype = match.arg(sievetype)
   if (is.logical(shade) && shade && is.null(gp))
     gp <- if (sievetype == "observed")
@@ -62,18 +67,16 @@ sieve.default <- function(x, condvars = NULL, gp = NULL,
     else
       shading_Friendly(interpolate = 0, line_col = "darkgray", eps = Inf, lty = "dotted")
 
-  if (inherits(x, "structable"))
+  if (is.structable(x)) {
+    if (is.null(direction) && is.null(split_vertical))
+      split_vertical <- attr(x, "split_vertical")
     x <- as.table(x)
+  }
+  if (is.null(split_vertical))
+    split_vertical <- FALSE
   
   dl <- length(dim(x))
-  if (!is.null(condvars)) {
-    if (is.character(condvars))
-      condvars <- match(condvars, names(dimnames(x)))
-    x <- aperm(x, c(condvars, seq(dl)[-condvars]))
-    if (is.null(spacing))
-      spacing <- spacing_conditional
-  }
-  
+
   ## splitting argument
   if (!is.null(direction))
     split_vertical <- direction == "v"
@@ -82,17 +85,27 @@ sieve.default <- function(x, condvars = NULL, gp = NULL,
   if (length(split_vertical) < dl)
     split_vertical <- rep(split_vertical, length.out = dl)
 
+  ## condvars
+  if (!is.null(condvars)) {
+    if (is.character(condvars))
+      condvars <- match(condvars, names(dimnames(x)))
+    x <- aperm(x, c(condvars, seq(dl)[-condvars]))
+    if (is.null(spacing))
+      spacing <- spacing_conditional
+  }
+  
   ## spacing argument
   if (is.null(spacing))
     spacing <- if (dl < 3) spacing_equal else spacing_increase
 
   strucplot(x,
             condvars = if (is.null(condvars)) NULL else length(condvars),
-            panel = struc_sieve(sievetype = sievetype),
+            core = struc_sieve(sievetype = sievetype),
             split_vertical = split_vertical,
             spacing = spacing,
             spacing_args = spacing_args,
             main = main,
+            sub = sub,
             shade = shade, 
             legend = legend,
             gp = gp,
@@ -123,7 +136,7 @@ struc_sieve <- function(sievetype = c("observed", "expected")) {
       else
         grid.layout(nrow = 2 * d - 1, heights = dist[idx])
       vproot <- viewport(layout.pos.col = col, layout.pos.row = row,
-                         layout = layout, name = substr(name, 1, nchar(name) - 1))
+                         layout = layout, name = remove_trailing_comma(name))
       
       ## next level: either create further splits, or final viewports
       name <- paste(name, dnn[i], "=", dn[[i]], ",", sep = "")
@@ -142,12 +155,12 @@ struc_sieve <- function(sievetype = c("observed", "expected")) {
       } else {
         if (v)
           function(m) viewport(layout.pos.col = col[m], layout.pos.row = row[m],
-                               name = substr(name[m], 1, nchar(name[m]) - 1),
+                               name = remove_trailing_comma(name[m]),
                                yscale = c(0, rowmargin),
                                xscale = c(0, colmargin * proptab(margin)[m]))
         else
           function(m) viewport(layout.pos.col = col[m], layout.pos.row = row[m],
-                               name = substr(name[m], 1, nchar(name[m]) - 1),
+                               name = remove_trailing_comma(name[m]),
                                yscale = c(0, rowmargin * proptab(margin)[m]),
                                xscale = c(0, colmargin))
       }
@@ -187,4 +200,4 @@ struc_sieve <- function(sievetype = c("observed", "expected")) {
     }
   }
 }
-class(struc_sieve) <- "panel_generator"
+class(struc_sieve) <- "grapcon_generator"
