@@ -5,14 +5,14 @@ spacing_equal <- function(sp = unit(0.5, "lines")) {
   if (!is.unit(sp)) sp <- unit(sp, "lines")
   function(d, condvars = NULL) lapply(d, function(x) unit.rep(sp, x - 1))
 }
-class(spacing_equal) <- "panel_generator"
+class(spacing_equal) <- "grapcon_generator"
 
 spacing_dimequal <- function(sp) {
   if (!is.unit(sp)) sp <- unit(sp, "lines")
   function(d, condvars = NULL)
     lapply(seq(along = d), function(i) unit.rep(sp[i], d[[i]] - 1))
 }
-class(spacing_dimequal) <- "panel_generator"
+class(spacing_dimequal) <- "grapcon_generator"
 
 spacing_increase <- function(start = unit(0.3, "lines"), rate = 1.5) {
   if (!is.unit(start)) start <- unit(start, "lines")
@@ -21,17 +21,15 @@ spacing_increase <- function(start = unit(0.3, "lines"), rate = 1.5) {
     lapply(seq(along = d), function(i) unit.rep(sp[i], d[[i]] - 1))
   }
 }
-class(spacing_increase) <- "panel_generator"
+class(spacing_increase) <- "grapcon_generator"
 
-spacing_doubledecker <- function(start = unit(0.3, "lines"), rate = 1.8)
-  function(d, condvars) {
-    if (length(d) < 3) {
-      start[2] <- 0
-      spacing_dimequal(start)(d, condvars)
-    } else
-      spacing_conditional(sp = 0, start = start, rate = rate)(d, condvars)
-  }
-class(spacing_doubledecker) <- "panel_generator"
+spacing_highlighting <- function(start = unit(0.2, "lines"), rate = 1.5) {
+ if (!is.unit(start)) start <- unit(start, "lines")
+  function(d, condvars = NULL)
+    c(spacing_increase(start, rate)(d, condvars)[-length(d)],
+      list(unit(rep(0, d[length(d)]), "lines")))
+}
+class(spacing_highlighting) <- "grapcon_generator"
 
 spacing_conditional <- function(sp = unit(0.5, "lines"),
                                 start = unit(2, "lines"), rate = 1.8) {
@@ -51,4 +49,4 @@ spacing_conditional <- function(sp = unit(0.5, "lines"),
     ret
   }
 }
-class(spacing_conditional) <- "panel_generator"
+class(spacing_conditional) <- "grapcon_generator"
