@@ -1,6 +1,6 @@
 ################################################################
 ### strucplot - generic plot framework for mosaic-like layouts
-### 2 panel functions are provided: struc_mosaic and struc_assoc
+### 2 core functions are provided: struc_mosaic and struc_assoc
 ################################################################
 
 strucplot <- function(## main parameters
@@ -13,28 +13,31 @@ strucplot <- function(## main parameters
                       residuals_type = c("Pearson", "deviance", "FT"),
                       
                       ## layout
-                      split_vertical = TRUE, 
+                      split_vertical = NULL, 
                       spacing = spacing_equal,
                       spacing_args = list(),
                       gp = NULL,
 		      gp_args = list(),   
                       labeling = labeling_border,
                       labeling_args = list(),
-                      panel = struc_mosaic,
-                      panel_args = list(),
+                      core = struc_mosaic,
+                      core_args = list(),
                       legend = NULL,
                       legend_args = list(),
                       
                       main = NULL,
                       sub = NULL,
-                      margins = rep.int(2.5, 4),
-                      legend_width = unit(5, "lines"),
+                      margins = unit(2.5, "lines"),
+                      title_margins = NULL,
+                      legend_width = NULL,
                       
                       ## control parameters
-                      title_gp = gpar(fontsize = 20),
+                      main_gp = gpar(fontsize = 20),
+                      sub_gp = gpar(fontsize = 15),
                       newpage = TRUE,
                       pop = TRUE,
-                      keep_aspect_ratio = TRUE
+                      keep_aspect_ratio = TRUE,
+                      ...
                       ) {
   ## default behaviour of shade
   if (is.null(shade)) shade <- !is.null(gp) || !is.null(expected)
@@ -42,6 +45,15 @@ strucplot <- function(## main parameters
   type <- match.arg(type)
   residuals_type <- match.arg(tolower(residuals_type), c("pearson", "deviance", "ft"))
 
+  ## convert structable object
+  if (is.structable(x)) {
+    if (is.null(split_vertical))
+      split_vertical <- attr(x, "split_vertical")
+    x <- as.table(x)
+  }
+  if (is.null(split_vertical))
+    split_vertical <- FALSE
+  
   ## table characteristics
   d <- dim(x)
   dl <- length(d)
@@ -96,7 +108,7 @@ strucplot <- function(## main parameters
 
   ## spacing
   if (is.function(spacing)) {
-    if (inherits(spacing, "panel_generator"))
+    if (inherits(spacing, "grapcon_generator"))
       spacing <- do.call("spacing", spacing_args)
     spacing <- spacing(d, condvars)
   }
@@ -107,14 +119,14 @@ strucplot <- function(## main parameters
     if (is.function(gp)) {
       if (is.null(legend) || (is.logical(legend) && legend))
         legend <- legend_resbased
-      gpfun <- if(inherits(gp, "panel_generator"))
+      gpfun <- if (inherits(gp, "grapcon_generator"))
         do.call("gp", c(list(x, residuals, expected, df), as.list(gp_args))) else gp
       gp <- gpfun(residuals)
     } else if (!is.null(legend) && !(is.logical(legend) && !legend))
       stop("gp argument must be a shading function for drawing a legend")
   } else {
     if(!is.null(gp)) {
-      warning("gp parameter ignored since shade=FALSE")
+      warning("gp parameter ignored since shade = FALSE")
       gp <- NULL
     }
   }
@@ -124,7 +136,11 @@ strucplot <- function(## main parameters
 
   ## recycle gpar values in the last dimension
   size <- prod(d)
-  FUN <- function(par) if (length(par) < size) aperm(array(par, dim = rev(d))) else par
+  FUN <- function(par) {
+    if (is.structable(par))
+      par <- as.table(par)
+    if (length(par) < size) aperm(array(par, dim = rev(d))) else par
+  }
   gp <- structure(lapply(gp, FUN), class = "gpar")
   
   ## set up page
@@ -134,12 +150,14 @@ strucplot <- function(## main parameters
     pushViewport(viewport(width = 1, height = 1, default.unit = "snpc"))
   
   pushViewport(vcdViewport(mar = margins,
+                           oma = title_margins,
                            legend = shade && !(is.null(legend) || is.logical(legend) && !legend),
-                           main = !is.null(main), sub = !is.null(sub), keep_aspect_ratio = keep_aspect_ratio,
+                           main = !is.null(main), sub = !is.null(sub),
+                           keep_aspect_ratio = keep_aspect_ratio,
                            legend_width = legend_width))
 
   ## legend
-  if (inherits(legend, "panel_generator"))
+  if (inherits(legend, "grapcon_generator"))
     legend <- do.call("legend", legend_args)
   if (shade && !is.null(legend) && !(is.logical(legend) && !legend)) {
     seekViewport("legend")
@@ -152,32 +170,34 @@ strucplot <- function(## main parameters
     seekViewport("main")
     if (is.logical(main) && main)
       main <- deparse(substitute(x))
-    grid.text(main, gp = title_gp)
+    grid.text(main, gp = main_gp)
   }
 
   if (!is.null(sub)) {
     seekViewport("sub")
-    grid.text(sub, gp = title_gp)
+    if (is.logical(sub) && sub && is.null(main))
+      sub <- deparse(substitute(x))
+    grid.text(sub, gp = sub_gp)
   }
 
   ## make plot
   seekViewport("plot")
   
-  if (inherits(panel, "panel_generator"))
-    panel <- do.call("panel", panel_args)
-  panel(residuals = residuals,
-        observed = if (type == "observed") x else expected,
-        expected = if (type == "observed") expected else x,
-        spacing = spacing,
-        gp = gp,
-        split_vertical = split_vertical)
+  if (inherits(core, "grapcon_generator"))
+    core <- do.call("core", core_args)
+  core(residuals = residuals,
+       observed = if (type == "observed") x else expected,
+       expected = if (type == "observed") expected else x,
+       spacing = spacing,
+       gp = gp,
+       split_vertical = split_vertical)
 
   upViewport(dl)
 
   ## labels
   if (!is.null(labeling)) {
-    if (inherits(labeling, "panel_generator"))
-      labeling <- do.call("labeling", labeling_args)
+    if (inherits(labeling, "grapcon_generator"))
+      labeling <- do.call("labeling", c(labeling_args, list(...)))
     labeling(dn, split_vertical, condvars)
   }
 
@@ -185,9 +205,6 @@ strucplot <- function(## main parameters
 
   seekViewport("base") 
   ## one more up if sandwich-mode
-  if (!is.null(main) || !is.null(sub) ##||
-##      (shade && !is.null(legend) && !(is.logical(legend) && !legend))
-      ) upViewport()
   if (pop) popViewport(1 + keep_aspect_ratio) else upViewport(1 + keep_aspect_ratio)
 
   ## return visualized table
@@ -197,79 +214,65 @@ strucplot <- function(## main parameters
 
 vcdViewport <- function(mar = rep.int(2.5, 4),
                         legend_width = unit(5, "lines"),
+                        oma = NULL,
                         legend = FALSE, main = FALSE, sub = FALSE,
                         keep_aspect_ratio = TRUE)
 {
+  ## process parameters
+  if (is.null(legend_width))
+    legend_width <- unit(5 * legend, "lines")
+  if (!is.unit(legend_width))
+    legend_width <- unit(legend_width, "lines")
+
+  if (legend && !main && !sub && keep_aspect_ratio) main <- sub <- TRUE
   mar <- if (!is.unit(mar))
     unit(pexpand(mar, 4, rep.int(2.5, 4), c("top","right","bottom","left")), "lines")
   else
     unit.rep(mar, length.out = 4)
-  if (!is.unit(legend_width))
-    legend_width <- unit(legend_width, "lines")
-  vpPlot <- vpStack(viewport(layout.pos.col = 2, layout.pos.row = 2),
+  if (is.null(oma)) {
+    space <- if (legend && keep_aspect_ratio)
+      legend_width + mar[2] + mar[4] - mar[1] - mar[3]
+    else unit(0, "lines")
+    oma <- if (main && sub)
+      max(unit(2, "lines"), 0.5 * space)
+    else if (main)
+      unit.c(max(unit(2, "lines"), space), unit(0, "lines"))
+    else if (sub)
+      unit.c(unit(0, "lines"), max(unit(2, "lines"), space))
+    else
+      0.5 * space
+  }
+  oma <- if (!is.unit(oma))
+    unit(pexpand(oma, 2, rep.int(2, 2), c("top","bottom")), "lines")
+  else
+    unit.rep(oma, length.out = 2)
+  
+  ## set up viewports
+  vpPlot <- vpStack(viewport(layout.pos.col = 2, layout.pos.row = 3),
                     viewport(width = 1, height = 1, name = "plot",
                              default.units = if (keep_aspect_ratio) "snpc" else "npc"))
-  vpMarginBottom <- viewport(layout.pos.col = 2, layout.pos.row = 3, name = "margin_bottom")
-  vpMarginLeft <- viewport(layout.pos.col = 1, layout.pos.row = 2, name = "margin_left")
-  vpMarginTop <- viewport(layout.pos.col = 2, layout.pos.row = 1, name = "margin_top")
-  vpMarginRight <- viewport(layout.pos.col = 3, layout.pos.row = 2, name = "margin_right")
-  vpCornerTL <- viewport(layout.pos.col = 1, layout.pos.row = 1, name = "corner_top_left")
-  vpCornerTR <- viewport(layout.pos.col = 3, layout.pos.row = 1, name = "corner_top_right")
-  vpCornerBL <- viewport(layout.pos.col = 1, layout.pos.row = 3, name = "corner_bottom_left")
-  vpCornerBR <- viewport(layout.pos.col = 3, layout.pos.row = 3, name = "corner_bottom_right")
+  vpMarginBottom <- viewport(layout.pos.col = 2, layout.pos.row = 4, name = "margin_bottom")
+  vpMarginLeft <- viewport(layout.pos.col = 1, layout.pos.row = 3, name = "margin_left")
+  vpMarginTop <- viewport(layout.pos.col = 2, layout.pos.row = 2, name = "margin_top")
+  vpMarginRight <- viewport(layout.pos.col = 3, layout.pos.row = 3, name = "margin_right")
+  vpCornerTL <- viewport(layout.pos.col = 1, layout.pos.row = 2, name = "corner_top_left")
+  vpCornerTR <- viewport(layout.pos.col = 3, layout.pos.row = 2, name = "corner_top_right")
+  vpCornerBL <- viewport(layout.pos.col = 1, layout.pos.row = 4, name = "corner_bottom_left")
+  vpCornerBR <- viewport(layout.pos.col = 3, layout.pos.row = 4, name = "corner_bottom_right")
 
-  if (legend) {
-    vpLegend <- viewport(layout.pos.col = 4, layout.pos.row = 2, name = "legend")
-    vpPval <- viewport(layout.pos.col = 4, layout.pos.row = 3, name = "pval")
-    vpBase <- viewport(layout.pos.row = 1 + main,
-                       layout = grid.layout(3, 4,
-                         widths = unit.c(mar[4], unit(1, "null"), mar[2], legend_width),
-                         heights = unit.c(mar[1], unit(1, "null"), mar[3])),
-                       name = "base")
-    vpPlotregion <- vpTree(vpBase, vpList(vpMarginBottom, vpMarginLeft, vpMarginTop,
-                                          vpMarginRight, vpPval, vpLegend,
-                                          vpCornerTL, vpCornerTR, vpCornerBL,
-                                          vpCornerBR, vpPlot))
-  } else {
-    vpBase <- viewport(layout.pos.row = 1 + main,
-                       layout = grid.layout(3, 3,
-                         widths = unit.c(mar[4], unit(1, "null"), mar[2]),
-                         heights = unit.c(mar[1], unit(1, "null"), mar[3])
-                         ),
-                       name = "base")
-    vpPlotregion <- vpTree(vpBase,
-                           vpList(vpMarginBottom, vpMarginLeft, vpMarginTop, vpMarginRight,
-                                  vpCornerTL, vpCornerTR, vpCornerBL, vpCornerBR, vpPlot))
-  }
-
-  ## main/sub-title, margins for legend layout
-  if (main || sub) {
-    vpTop <- viewport(layout.pos.row = 1, name = "main")
-    vpSub <- viewport(layout.pos.row = 2 + main, name = "sub")
-    
-    sandwich <-
-## no additional space when keep_aspect_ratio = F
-#       if (legend) {
-#       space <- max(legend_width + mar[2] + mar[4] - mar[1] - mar[3],
-#                    unit((main + sub) * 2, "lines"))
-#       vplist <- vpList(vpTop, vpPlotregion, vpSub)
-#       viewport(layout = grid.layout(3, 1,
-#                  height = unit.c(0.5 * space, unit(1, "null"), 0.5 * space)))
-#     } else
-    if (main && sub) {
-      vplist <- vpList(vpTop, vpPlotregion, vpSub)
-      viewport(layout = grid.layout(3, 1,
-                 height = unit.c(unit(2, "lines"), unit(1, "null"), unit(2, "lines"))))
-    } else if (main) {
-      vplist <- vpList(vpTop, vpPlotregion)
-      viewport(layout = grid.layout(2, 1,
-                 height = unit.c(unit(2, "lines"), unit(1, "null"))))
-    } else {
-      vplist <- vpList(vpPlotregion, vpSub)
-      viewport(layout = grid.layout(2, 1,
-                 height = unit.c(unit(1, "null"), unit(2, "lines"))))
-    }
-
-    vpTree(sandwich, vplist)
-  } else vpPlotregion
+  vpLegend <- viewport(layout.pos.col = 4, layout.pos.row = 3, name = "legend")
+  vpLegendTop <- viewport(layout.pos.col = 4, layout.pos.row = 2, name = "legend_top")
+  vpLegendSub <- viewport(layout.pos.col = 4, layout.pos.row = 4, name = "legend_sub")
+  vpBase <- viewport(layout = grid.layout(5, 4,
+                       widths = unit.c(mar[4], unit(1, "null"), mar[2], legend_width),
+                       heights = unit.c(oma[1], mar[1], unit(1, "null"), mar[3], oma[2])),
+                     name = "base")
+  vpMain <- viewport(layout.pos.col = 1:4, layout.pos.row = 1, name = "main")
+  vpSub <- viewport(layout.pos.col = 1:4, layout.pos.row = 5, name = "sub")
+  
+  vpTree(vpBase, vpList(vpMain, vpMarginBottom, vpMarginLeft, vpMarginTop,
+                        vpMarginRight, vpLegendTop, vpLegend,
+                        vpLegendSub, vpCornerTL, vpCornerTR,
+                        vpCornerBL, vpCornerBR, vpPlot, vpSub))
 }
+
