@@ -5,7 +5,7 @@ structable <- function(x, ...)
   UseMethod("structable")
 
 structable.formula <- function(formula, data = NULL, direction = NULL,
-                               split_vertical = FALSE, ..., subset, na.action) {
+                               split_vertical = NULL, ..., subset, na.action) {
     if (missing(formula) || !inherits(formula, "formula")) 
         stop("formula is incorrect or missing")
 
@@ -14,7 +14,15 @@ structable.formula <- function(formula, data = NULL, direction = NULL,
 
     if (!is.null(direction))
       split_vertical <- direction == "v"
-        
+
+    if (is.structable(data)) {
+      split_vertical <- attr(data, "split_vertical")
+      data <- as.table(data)
+    }
+    
+    if (is.null(split_vertical))
+      split_vertical <- FALSE
+    
     ## only rhs present without `.' in lhs => xtabs-interface
     if (length(formula) != 3) {
       if (formula[[1]] == "~") {
@@ -154,54 +162,154 @@ structable.default <- function(..., direction = NULL, split_vertical = FALSE) {
   ret
 }
 
-"[[.structable" <- function(x, ...) {
+"[.structable" <- function(x, ...) {
+  args <- if (nargs() < 3)
+    list(..1)
+  else
+    list(..1, ..2)
+  
   ## handle one-arg cases
   if (nargs() < 3)
-    if (length(..1) > 1)
-      ## resolve calls like x[[c(1,2)]]
-      return(x[[ ..1[1] ]] [[ ..1[-1] ]])
+    if (length(args[[1]]) > 1)
+      ## resolve calls like x[c(1,2)]
+      return(x[ args[[1]][1] ] [ args[[1]][-1] ])
     else
-      ## resolve x[[foo]] 
-      return(if (attr(x, "split_vertical")[1]) x[[,..1]] else x[[..1,]])
+      ## resolve x[foo] 
+      return(if (attr(x, "split_vertical")[1]) x[,args[[1]] ] else x[args[[1]],])
 
-  ## handle calls like x[[c(1,2), c(3,4)]]
-  if (length(..1) > 1 && length(..2) > 1)
-    return(x[[ ..1[1], ..2[[1]] ]] [[ ..1[-1], ..2[-1] ]])
-  ## handle calls like x[[c(1,2), 3]]
-  if (length(..1) > 1)
-    return(x[[ ..1[1], ..2 ]] [[ ..1[-1], ]])
-  ## handle calls like x[[1, c(1,3)]]
-  if (length(..2) > 1)
-    return(x[[ ..1, ..2[[1]] ]] [[ , ..2[-1] ]])
-
-  ## final cases like x[[1,2]] or x[[1,]] or x[[,1]]
-  rv <- attr(x, "dnames")[!attr(x, "split_vertical")]
-  cv <- attr(x, "dnames")[attr(x, "split_vertical")]
-
-  if (!is.symbol(..1)) rstep <- dim(x)[1] / length(rv[[1]])
-  if (!is.symbol(..2)) cstep <- dim(x)[2] / length(cv[[1]])
-
-  ret <- x[if (!is.symbol(..1)) (1 + (..1 - 1) * rstep) : (..1 * rstep) else 1:nrow(x),
-           if (!is.symbol(..2)) (1 + (..2 - 1) * cstep) : (..2 * cstep) else 1:ncol(x),
-           drop = FALSE
-           ]
-
-  attr(ret, "split_vertical") <- attr(x, "split_vertical")
-  attr(ret, "dnames") <- attr(x, "dnames")
+  ## handle calls like x[c(1,2), c(3,4)]
+  if (length(args[[1]]) > 1 && length(args[[2]]) > 1)
+    return(x[ args[[1]][1], args[[2]][1] ] [ args[[1]][-1], args[[2]][-1] ])
   
-  if (!is.symbol(..1)) {
-    i <- which(!attr(ret, "split_vertical"))[1]
-    attr(ret, "split_vertical") <- attr(ret, "split_vertical")[-i]
-    attr(ret, "dnames") <- attr(ret, "dnames")[-i]
+  ## handle calls like x[c(1,2), 3]
+  if (length(args[[1]]) > 1)
+    return(x[ args[[1]][1], args[[2]] ] [ args[[1]][-1], ])
+  
+  ## handle calls like x[1, c(1,3)]
+  if (length(args[[2]]) > 1)
+    return(x[ args[[1]], args[[2]][1] ] [ , args[[2]][-1] ])
+
+  ## final cases like x[1,2] or x[1,] or x[,1]
+  dnames <- attr(x, "dnames")
+  split <- attr(x, "split_vertical")
+  rv <- dnames[!split]
+  cv <- dnames[split]
+
+  lsym <- is.symbol(args[[1]])
+  rsym <- is.symbol(args[[2]])
+  if (!lsym) {
+    rstep <- dim(unclass(x))[1] / length(rv[[1]])
+    if (is.character(args[[1]]))
+      args[[1]] <- match(args[[1]], rv[[1]])
+  }
+  if (!rsym) {
+    cstep <- dim(unclass(x))[2] / length(cv[[1]])
+    if (is.character(args[[2]]))
+      args[[2]] <- match(args[[2]], cv[[1]])
+  }
+
+  lind <- if (!lsym)
+    (1 + (args[[1]] - 1) * rstep) : (args[[1]] * rstep)
+  else
+    1:nrow(unclass(x))
+  rind <- if (!rsym)
+    (1 + (args[[2]] - 1) * cstep) : (args[[2]] * cstep)
+  else
+    1:ncol(unclass(x))
+  ret <- unclass(x)[lind, rind, drop = FALSE]
+
+  if (!lsym) {
+    i <- which(!split)[1]
+    split <- split[-i]
+    dnames <- dnames[-i]
   }
     
-
-  if (!is.symbol(..2)) {
-    i <- which(attr(ret, "split_vertical"))[1]
-    attr(ret, "split_vertical") <- attr(ret, "split_vertical")[-i]
-    attr(ret, "dnames") <- attr(ret, "dnames")[-i]
+  if (!rsym) {
+    i <- which(split)[1]
+    split <- split[-i]
+    dnames <- dnames[-i]
   }
 
+  attr(ret, "split_vertical") <- split
+  attr(ret, "dnames") <- dnames
+  
+  ## add dimension attributes in ftable-format
+  attr(ret, "col.vars") <- dnames[split]
+  attr(ret, "row.vars") <- dnames[!split]
+
+  class(ret) <- class(x)
+  ret
+}
+
+"[<-.structable" <- function(x, ..., value) {
+  args <- if (nargs() < 4)
+    list(..1)
+  else
+    list(..1, ..2)
+  
+  ## handle one-arg cases
+  if (nargs() < 4)
+    return(if (length(args[[1]]) > 1)
+               ## resolve calls like x[c(1,2)]
+               Recall(x, args[[1]][1],
+                      value = Recall(x[ args[[1]][1] ], args[[1]][-1], value = value))
+           else
+               ## resolve x[foo] 
+               if (attr(x, "split_vertical")[1])
+                   Recall(x,,args[[1]], value = value)
+               else
+                   Recall(x,args[[1]],, value = value)
+           )
+  
+  ## handle calls like x[c(1,2), c(3,4)]
+  if (length(args[[1]]) > 1 && length(args[[2]]) > 1)
+    return(Recall(x, args[[1]][1], args[[2]][1],
+                  value = Recall(x[ args[[1]][1], args[[2]][1] ],
+                    args[[1]][-1], args[[2]][-1], value = value)))
+  
+  ## handle calls like x[c(1,2), 3]
+  if (length(args[[1]]) > 1)
+    return(Recall(x, args[[1]][1], args[[2]],
+                  value = Recall(x[ args[[1]][1], args[[2]] ],
+                    args[[1]][-1], ,value = value)))
+  
+  ## handle calls like x[1, c(1,3)]
+  if (length(args[[2]]) > 1)
+    return(Recall(x, args[[1]], args[[2]][1],
+                  value = Recall(x[ args[[1]], args[[2]][1] ],,
+                    args[[2]][-1], value = value)))
+
+  ## final cases like x[1,2] or x[1,] or x[,1]
+  dnames <- attr(x, "dnames")
+  split <- attr(x, "split_vertical")
+  rv <- dnames[!split]
+  cv <- dnames[split]
+
+  lsym <- is.symbol(args[[1]])
+  rsym <- is.symbol(args[[2]])
+  if (!lsym) {
+    rstep <- dim(unclass(x))[1] / length(rv[[1]])
+    if (is.character(args[[1]]))
+      args[[1]] <- match(args[[1]], rv[[1]])
+  }
+  if (!rsym) {
+    cstep <- dim(unclass(x))[2] / length(cv[[1]])
+    if (is.character(args[[2]]))
+      args[[2]] <- match(args[[2]], cv[[1]])
+  }
+
+  lind <- if (!lsym)
+    (1 + (args[[1]] - 1) * rstep) : (args[[1]] * rstep)
+  else
+    1:nrow(unclass(x))
+  rind <- if (!rsym)
+    (1 + (args[[2]] - 1) * cstep) : (args[[2]] * cstep)
+  else
+    1:ncol(unclass(x))
+  ret <- unclass(x)
+
+  ret[lind, rind] <- value
+
   class(ret) <- class(x)
   ret
 }
@@ -215,3 +323,29 @@ as.table.structable <- function(x, ...) {
 plot.structable <- function(x, ...)
   mosaic(x, ...)
 
+t.structable <- function(x) {
+  ret <- t.default(x)
+  attr(ret, "split_vertical") <- !attr(ret, "split_vertical")
+  hold <- attr(ret, "row.vars")
+  attr(ret, "row.vars") = attr(ret, "col.vars")
+  attr(ret, "col.vars") = hold
+  ret
+}
+
+is.structable <- function(x)
+  inherits(x, "structable")
+
+dim.structable <- function(x)
+  as.integer(sapply(attr(x, "dnames"), length))
+
+print.structable <- function(x, ...) {
+  class(x) <- "ftable"
+  NextMethod("print", object = x)
+}
+
+dimnames.structable <- function(x) attr(x,"dnames")
+
+as.vector.structable <- function(x, ...)
+  as.vector(as.table(x), ...)
+  
+as.matrix.structable <- function(x) matrix(as.vector(unclass(x)), ncol = attr(x, "dim")[2])
